# Standard Library
from types import SimpleNamespace

# Django
from django.template.loader import render_to_string
from django.test import SimpleTestCase


class WeightedAnswersTemplateRenderingTests(SimpleTestCase):
    def _quiz(self, show_answers_corrections_text):
        return SimpleNamespace(
            name="Quiz test",
            show_score_summary=True,
            show_answers_corrections_text=show_answers_corrections_text,
        )

    def test_weighted_summary_displays_explanatory_text_when_enabled(self):
        context = {
            "quiz": self._quiz(True),
            "conclusion": {"points": 4, "description": "<p>ok</p>"},
            "final_quiz": [
                {
                    "question": {"label": "Q1"},
                    "answers": ["R1"],
                    "question_explanation_text": "<p>Texte info</p>",
                }
            ],
            "relaunch_button": False,
        }

        rendered = render_to_string(
            "django_easy_quiz/weighted_answers_form.html",
            context,
        )

        self.assertIn("Texte info", rendered)
        self.assertIn("quiz-answer-explanatory-text", rendered)

    def test_weighted_summary_hides_explanatory_text_when_disabled(self):
        context = {
            "quiz": self._quiz(False),
            "conclusion": {"points": 4, "description": "<p>ok</p>"},
            "final_quiz": [
                {
                    "question": {"label": "Q1"},
                    "answers": ["R1"],
                    "question_explanation_text": "<p>Texte info</p>",
                }
            ],
            "relaunch_button": False,
        }

        rendered = render_to_string(
            "django_easy_quiz/weighted_answers_form.html",
            context,
        )

        self.assertNotIn("Texte info", rendered)
        self.assertNotIn("quiz-answer-explanatory-text", rendered)

    def test_weighted_summary_supports_legacy_string_answers(self):
        context = {
            "quiz": self._quiz(True),
            "conclusion": {"points": 2, "description": "<p>ok</p>"},
            "final_quiz": [
                {
                    "question": {"label": "Q legacy"},
                    "answers": ["<b>Ancienne réponse</b>"],
                }
            ],
            "relaunch_button": False,
        }

        rendered = render_to_string(
            "django_easy_quiz/weighted_answers_form.html",
            context,
        )

        self.assertIn("Ancienne réponse", rendered)

    def test_weighted_summary_partial_handles_selected_and_not_selected_answers(self):
        rendered = render_to_string(
            "django_easy_quiz/partials/weighted_summary_answers.html",
            {
                "fieldset_index": 2,
                "answer_entries": [
                    {"id": 10, "label": "Yes", "selected": True},
                    {"id": 11, "label": "No", "selected": False},
                ],
            },
        )

        self.assertIn("wa_sum_2_1", rendered)
        self.assertIn("wa_sum_2_2", rendered)
        self.assertIn("Yes", rendered)
        self.assertIn("No", rendered)
        self.assertIn("quiz-answer--not-selected", rendered)

    def test_weighted_summary_partial_supports_legacy_text_and_empty_state(self):
        rendered_legacy = render_to_string(
            "django_easy_quiz/partials/weighted_summary_answers.html",
            {
                "fieldset_index": 1,
                "answer_entries": ["<b>Legacy value</b>"],
            },
        )
        rendered_empty = render_to_string(
            "django_easy_quiz/partials/weighted_summary_answers.html",
            {
                "fieldset_index": 1,
                "answer_entries": [],
            },
        )

        self.assertIn("Legacy value", rendered_legacy)
        self.assertIn("You haven't selected any answer.", rendered_empty)


class PdfTemplateRenderingTests(SimpleTestCase):
    def test_pdf_template_displays_explanatory_text_when_enabled(self):
        context = {
            "quiz_type": "weighed_answers_quiz",
            "conclusion": {"points": 5, "description": "<p>Done</p>"},
            "final_quiz": [
                {
                    "question": {"label": "Q1"},
                    "answers": ["A1"],
                    "question_explanation_text": "<p>Info PDF</p>",
                }
            ],
            "final_quiz_show_answers_corrections_text": True,
            "logo": "logo.png",
        }

        rendered = render_to_string("django_easy_quiz/quiz_pdf.html", context)

        self.assertIn("Info PDF", rendered)
        self.assertIn("quiz-answer-explanatory-text", rendered)

    def test_pdf_template_supports_legacy_string_answers(self):
        context = {
            "quiz_type": "weighed_answers_quiz",
            "conclusion": {"points": 3, "description": "<p>Done</p>"},
            "final_quiz": [
                {
                    "question": {"label": "Q1"},
                    "answers": ["<i>Answer legacy</i>"],
                }
            ],
            "final_quiz_show_answers_corrections_text": False,
            "logo": "logo.png",
        }

        rendered = render_to_string("django_easy_quiz/quiz_pdf.html", context)

        self.assertIn("Answer legacy", rendered)



# Standard Library
import sys
from types import SimpleNamespace
from unittest.mock import Mock, patch
from uuid import uuid4

# Django
from django.test import RequestFactory, SimpleTestCase

# Project
from django_easy_quiz.views.misc import DisplayPdfView


class _FakeFilterResult:
    def __init__(self, saved_quiz):
        self._saved_quiz = saved_quiz

    def first(self):
        return self._saved_quiz


class _FakeHTML:
    last_call = None

    def __init__(self, string, base_url):
        _FakeHTML.last_call = {"string": string, "base_url": base_url}

    def write_pdf(self):
        return b"%PDF-1.4 fake"


class DisplayPdfViewTests(SimpleTestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.identifier = uuid4()

    def test_pdf_view_weighted_quiz_includes_explanatory_flag(self):
        request = self.factory.get("/pdf/test?name=quiz_result")
        saved_quiz = SimpleNamespace(
            interpretation_quiz=None,
            weighted_answers_quiz=SimpleNamespace(show_answers_corrections_text=True),
            answers=[
                {
                    "question": {"label": "Q1"},
                    "answers": ["A1"],
                    "question_explanation_text": "<p>Info Q1</p>",
                }
            ],
            conclusion={"points": 4, "description": "<p>Done</p>"},
        )
        render_mock = Mock(return_value="<html>ok</html>")

        with patch(
            "django_easy_quiz.views.misc.SavedQuiz.objects.filter",
            return_value=_FakeFilterResult(saved_quiz),
        ), patch(
            "django_easy_quiz.views.misc.render_to_string",
            render_mock,
        ), patch.dict(
            sys.modules, {"weasyprint": SimpleNamespace(HTML=_FakeHTML)}
        ):
            response = DisplayPdfView().get(request, identifier=self.identifier)

        self.assertEqual(response["Content-Type"], "application/pdf")
        self.assertEqual(response["Content-Disposition"], "filename=quiz_result.pdf")
        context = render_mock.call_args[0][1]
        self.assertEqual(context["quiz_type"], "weighed_answers_quiz")
        self.assertTrue(context["final_quiz_show_answers_corrections_text"])

    def test_pdf_view_interpretation_quiz_disables_explanatory_flag_and_default_name(
        self,
    ):
        request = self.factory.get("/pdf/test")
        saved_quiz = SimpleNamespace(
            interpretation_quiz=SimpleNamespace(id=1),
            weighted_answers_quiz=None,
            answers=[{"question": {"label": "Q1"}, "answer": "■ - A"}],
            conclusion="Interpretation text",
        )
        render_mock = Mock(return_value="<html>ok</html>")

        with patch(
            "django_easy_quiz.views.misc.SavedQuiz.objects.filter",
            return_value=_FakeFilterResult(saved_quiz),
        ), patch(
            "django_easy_quiz.views.misc.render_to_string",
            render_mock,
        ), patch.dict(
            sys.modules, {"weasyprint": SimpleNamespace(HTML=_FakeHTML)}
        ):
            response = DisplayPdfView().get(request, identifier=self.identifier)

        self.assertEqual(response["Content-Disposition"], "filename=document.pdf")
        context = render_mock.call_args[0][1]
        self.assertEqual(context["quiz_type"], "interpretation_quiz")
        self.assertFalse(context["final_quiz_show_answers_corrections_text"])



SECRET_KEY = "django-easy-quiz-tests"
DEBUG = True
USE_TZ = True
USE_I18N = True
LANGUAGE_CODE = "en"
LANGUAGES = (
    ("en", "English"),
    ("fr", "French"),
)

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ":memory:",
    }
}

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django_easy_quiz",
]

ROOT_URLCONF = "django_easy_quiz.tests.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.request",
            ],
        },
    }
]

MIGRATION_MODULES = {
    "django_easy_quiz": None,
}



# Standard Library
from unittest.mock import patch

# Django
from django.test import TestCase

# Project
from django_easy_quiz.models import (
    SavedQuiz,
    WeightedAnswersQuiz,
    WeightedAnswersQuizAnswer,
    WeightedAnswersQuizConclusion,
    WeightedAnswersQuizQuestion,
)


class WeightedAnswersQuizViewTests(TestCase):
    def _create_quiz(self):
        quiz = WeightedAnswersQuiz.objects.create(
            name="Weighted",
            description="<p>Desc</p>",
            show_answers_corrections_text=True,
        )
        question = WeightedAnswersQuizQuestion.objects.create(
            quiz=quiz,
            label="Q1",
            multiple_answers=True,
            question_explanation_text="<p>Question info</p>",
        )
        a1 = WeightedAnswersQuizAnswer.objects.create(
            question=question, label="A1", points=2, order=1
        )
        WeightedAnswersQuizAnswer.objects.create(
            question=question, label="A2", points=3, order=2
        )
        WeightedAnswersQuizConclusion.objects.create(
            quiz=quiz,
            min_points=0,
            max_points=10,
            description="<p>Conclusion</p>",
        )
        return quiz, question, a1

    def test_post_saves_new_answers_format_and_points(self):
        quiz, question, a1 = self._create_quiz()
        a2 = question.answers.get(order=2)
        saved_quiz = SavedQuiz.objects.create(quiz_type="Weighted", quiz_ended=False)

        with patch("django_easy_quiz.views.weightedanswers.SAVE_QUIZZES_RESULTS", True):
            response = self.client.post(
                f"/weighted-answers/{quiz.id}/",
                data={
                    f"question_{question.id}": [str(a1.id), str(a2.id)],
                    "saved_quiz": str(saved_quiz.id),
                },
            )

        self.assertEqual(response.status_code, 200)
        saved_quiz.refresh_from_db()
        self.assertTrue(saved_quiz.quiz_ended)
        self.assertEqual(saved_quiz.conclusion["points"], 5)
        selected_labels = [
            a["label"] for a in saved_quiz.answers[0]["answers"] if a["selected"]
        ]
        self.assertEqual(selected_labels, ["A1", "A2"])
        self.assertEqual(
            saved_quiz.answers[0]["question_explanation_text"], "<p>Question info</p>"
        )

    def test_post_without_matching_conclusion_fallback(self):
        quiz = WeightedAnswersQuiz.objects.create(name="Weighted 2", description="Desc")
        question = WeightedAnswersQuizQuestion.objects.create(quiz=quiz, label="Q2")
        a1 = WeightedAnswersQuizAnswer.objects.create(
            question=question, label="A1", points=99, order=1
        )
        WeightedAnswersQuizAnswer.objects.create(
            question=question, label="A2", points=1, order=2
        )
        saved_quiz = SavedQuiz.objects.create(quiz_type="Weighted", quiz_ended=False)

        with patch("django_easy_quiz.views.weightedanswers.SAVE_QUIZZES_RESULTS", True):
            response = self.client.post(
                f"/weighted-answers/{quiz.id}/",
                data={
                    f"question_{question.id}": str(a1.id),
                    "saved_quiz": str(saved_quiz.id),
                },
            )

        self.assertEqual(response.status_code, 200)
        saved_quiz.refresh_from_db()
        self.assertEqual(saved_quiz.conclusion["points"], 99)
        self.assertEqual(saved_quiz.conclusion["description"], "")

    def test_get_summary_from_querystring(self):
        quiz, question, a1 = self._create_quiz()
        response = self.client.get(
            f"/weighted-answers/{quiz.id}/?question={question.id}.{a1.id}"
        )
        self.assertEqual(response.status_code, 200)
        first = response.context["quiz_answers"][0]["answers"][0]
        self.assertEqual(first["label"], "A1")
        self.assertTrue(first["selected"])



# Standard Library
from datetime import datetime
from io import BytesIO
from types import SimpleNamespace
from uuid import uuid4

# Django
from django.test import SimpleTestCase
from django.utils.translation import override

# Third party
from openpyxl import load_workbook

# Project
from django_easy_quiz.exporters.saved_quiz_export import xlsx_response


class SavedQuizExportTests(SimpleTestCase):
    def _saved_quiz(self, **kwargs):
        defaults = {
            "id": 1,
            "uuid": uuid4(),
            "started_quiz": datetime(2026, 1, 1, 10, 0, 0),
            "quiz_ended": True,
            "quiz_type": "Personality Quiz",
            "weighted_answers_quiz_id": None,
            "interpretation_quiz_id": None,
            "weighted_answers_quiz": None,
            "interpretation_quiz": None,
            "conclusion": {},
            "answers": [],
            "more_infos": [],
        }
        defaults.update(kwargs)
        return SimpleNamespace(**defaults)

    def _quiz(self, quiz_id, name, create_date, show_answers_corrections_text=False):
        return SimpleNamespace(
            id=quiz_id,
            name=name,
            create_date=create_date,
            show_answers_corrections_text=show_answers_corrections_text,
        )

    def _parse_workbook(self, response):
        return load_workbook(BytesIO(response.content))

    def test_xlsx_excludes_not_ended_quizzes(self):
        saved_quiz = self._saved_quiz(
            weighted_answers_quiz_id=3,
            weighted_answers_quiz=self._quiz(3, "WA 3", datetime(2026, 1, 2, 10, 0, 0)),
            conclusion={"points": 7, "description": "<p>Great!</p>"},
            answers=[
                {
                    "question": {"label": "Favorite fruits"},
                    "answers": ["<b>Apple</b>", "Pear"],
                }
            ],
            more_infos=[{"question": "Email", "answer": "john@example.com"}],
        )
        unfinished = self._saved_quiz(
            id=99,
            quiz_ended=False,
            weighted_answers_quiz_id=3,
            weighted_answers_quiz=self._quiz(3, "WA 3", datetime(2026, 1, 2, 10, 0, 0)),
            answers=[{"question": {"label": "Should not exist"}, "answers": ["X"]}],
        )

        with override("fr"):
            response = xlsx_response([saved_quiz, unfinished], "saved_quizzes.xlsx")
        workbook = self._parse_workbook(response)
        sheet = workbook["#3 - WA 3"]
        self.assertEqual(sheet.max_row, 2)
        self.assertEqual(sheet["A2"].value, 1)

    def test_xlsx_creates_one_sheet_per_selected_quiz(self):
        wa_old = self._quiz(1, "WA 1", datetime(2026, 1, 1, 10, 0, 0))
        wa_new = self._quiz(2, "WA 2", datetime(2026, 1, 3, 10, 0, 0))
        saved_quiz = self._saved_quiz(
            id=10,
            weighted_answers_quiz_id=1,
            weighted_answers_quiz=wa_old,
            answers=[
                {"question": {"label": "Question 1"}, "answers": ["A"]},
            ],
        )
        saved_quiz_new = self._saved_quiz(
            id=11,
            weighted_answers_quiz_id=2,
            weighted_answers_quiz=wa_new,
            answers=[{"question": {"label": "Question 2"}, "answers": ["B"]}],
        )

        with override("fr"):
            response = xlsx_response([saved_quiz, saved_quiz_new], "saved_quizzes.xlsx")
        workbook = self._parse_workbook(response)

        self.assertEqual(workbook.sheetnames, ["#2 - WA 2", "#1 - WA 1"])

    def test_xlsx_header_style_and_question_labels(self):
        wa_quiz = self._quiz(
            7,
            "WA 7",
            datetime(2026, 1, 4, 10, 0, 0),
            show_answers_corrections_text=True,
        )
        saved_quiz = self._saved_quiz(
            weighted_answers_quiz_id=7,
            weighted_answers_quiz=wa_quiz,
            quiz_type="WA Seven",
            answers=[{"question": {"label": "Favorite fruits"}, "answers": ["Apple"]}],
        )

        with override("fr"):
            response = xlsx_response([saved_quiz], "saved_quizzes.xlsx")
        workbook = self._parse_workbook(response)
        sheet = workbook["#7 - WA 7"]

        header_values = [cell.value for cell in sheet[1]]
        self.assertIn("Question n°1 : Favorite fruits", header_values)
        self.assertIn("Réponse question n°1", header_values)
        self.assertIn("Texte explicatif question n°1", header_values)
        self.assertTrue(all(cell.font.bold for cell in sheet[1]))

        # Width is capped to ~600px equivalent
        first_letter = sheet[1][0].column_letter
        self.assertLessEqual(sheet.column_dimensions[first_letter].width, 85.0)

    def test_xlsx_sheet_title_is_sanitized(self):
        quiz = self._quiz(
            12, "Au travail : quels risques pour", datetime(2026, 1, 4, 10, 0, 0)
        )
        saved_quiz = self._saved_quiz(
            weighted_answers_quiz_id=12,
            weighted_answers_quiz=quiz,
            answers=[{"question": {"label": "Q1"}, "answers": ["A1"]}],
        )

        response = xlsx_response([saved_quiz], "saved_quizzes.xlsx")
        workbook = self._parse_workbook(response)
        self.assertEqual(workbook.sheetnames[0], "#12 - Au travail quels risques")

    def test_xlsx_more_info_headers_use_question_text_not_slug(self):
        wa_quiz = self._quiz(5, "Main quiz", datetime(2026, 1, 5, 10, 0, 0))
        saved_quiz = self._saved_quiz(
            weighted_answers_quiz_id=5,
            weighted_answers_quiz=wa_quiz,
            answers=[{"question": {"label": "Q1"}, "answers": ["A"]}],
            more_infos=[
                {
                    "question": "Dans quel département vivez-vous ?",
                    "answer": "26",
                }
            ],
        )
        with override("fr"):
            response = xlsx_response([saved_quiz], "saved_quizzes.xlsx")
        workbook = self._parse_workbook(response)
        sheet = workbook["#5 - Main quiz"]
        header_values = [cell.value for cell in sheet[1]]
        self.assertIn("Dans quel département vivez-vous ?", header_values)
        self.assertNotIn("dans-quel-departement-vivez-vous", header_values)

    def test_xlsx_exports_answers_explanatory_text(self):
        wa_quiz = self._quiz(
            13,
            "WA 13",
            datetime(2026, 1, 4, 10, 0, 0),
            show_answers_corrections_text=True,
        )
        saved_quiz = self._saved_quiz(
            weighted_answers_quiz_id=13,
            weighted_answers_quiz=wa_quiz,
            answers=[
                {
                    "question": {"label": "Question 1"},
                    "answers": ["<b>1</b>", "2"],
                    "question_explanation_text": "<p>Info 1</p>",
                }
            ],
        )

        with override("fr"):
            response = xlsx_response([saved_quiz], "saved_quizzes.xlsx")
        workbook = self._parse_workbook(response)
        sheet = workbook["#13 - WA 13"]

        header_values = [cell.value for cell in sheet[1]]
        answer_info_index = header_values.index("Texte explicatif question n°1") + 1
        self.assertEqual(sheet.cell(row=2, column=answer_info_index).value, "Info 1")

    def test_xlsx_interpretation_quiz_uses_answer_field(self):
        interpretation_quiz = self._quiz(
            21, "Interpretation 21", datetime(2026, 1, 6, 10, 0, 0)
        )
        saved_quiz = self._saved_quiz(
            interpretation_quiz_id=21,
            interpretation_quiz=interpretation_quiz,
            answers=[
                {
                    "question": {"label": "Interpretation question"},
                    "answer": "■ - Option A",
                }
            ],
        )

        with override("en"):
            response = xlsx_response([saved_quiz], "saved_quizzes.xlsx")
        workbook = self._parse_workbook(response)
        sheet = workbook["#21 - Interpretation 21"]

        headers = [cell.value for cell in sheet[1]]
        answer_index = headers.index("Answer question #1") + 1
        self.assertEqual(sheet.cell(row=2, column=answer_index).value, "■ - Option A")

    def test_xlsx_fallback_sheet_when_no_answers(self):
        wa_quiz = self._quiz(30, "WA Empty", datetime(2026, 1, 6, 10, 0, 0))
        saved_quiz = self._saved_quiz(
            weighted_answers_quiz_id=30,
            weighted_answers_quiz=wa_quiz,
            answers=[],
        )

        with override("en"):
            response = xlsx_response([saved_quiz], "saved_quizzes.xlsx")
        workbook = self._parse_workbook(response)

        self.assertEqual(workbook.sheetnames, ["saved_quizzes"])
        sheet = workbook["saved_quizzes"]
        self.assertEqual(sheet.max_row, 1)
        self.assertEqual(sheet["A1"].value, "ID")

    def test_xlsx_strips_html_and_images_in_answers(self):
        wa_quiz = self._quiz(
            31,
            "WA HTML",
            datetime(2026, 1, 6, 10, 0, 0),
            show_answers_corrections_text=True,
        )
        saved_quiz = self._saved_quiz(
            weighted_answers_quiz_id=31,
            weighted_answers_quiz=wa_quiz,
            answers=[
                {
                    "question": {"label": "<b>Question</b>"},
                    "answers": ['<img src="/media/a.png" alt="pic">Answer <b>A</b>'],
                    "question_explanation_text": "<p>Info <img src='/media/i.png' /> detail</p>",
                }
            ],
        )

        with override("en"):
            response = xlsx_response([saved_quiz], "saved_quizzes.xlsx")
        workbook = self._parse_workbook(response)
        sheet = workbook["#31 - WA HTML"]
        headers = [cell.value for cell in sheet[1]]
        answer_index = headers.index("Answer question #1") + 1
        answer_info_index = headers.index("Explanatory text question #1") + 1

        self.assertEqual(sheet.cell(row=2, column=answer_index).value, "Answer A")
        self.assertEqual(
            sheet.cell(row=2, column=answer_info_index).value, "Info  detail"
        )

    def test_xlsx_hides_explanatory_columns_when_quiz_option_disabled(self):
        wa_quiz = self._quiz(41, "WA 41", datetime(2026, 1, 6, 10, 0, 0))
        saved_quiz = self._saved_quiz(
            weighted_answers_quiz_id=41,
            weighted_answers_quiz=wa_quiz,
            answers=[
                {
                    "question": {"label": "Question 1"},
                    "answers": ["A"],
                    "question_explanation_text": "<p>Info 1</p>",
                }
            ],
        )

        with override("en"):
            response = xlsx_response([saved_quiz], "saved_quizzes.xlsx")
        workbook = self._parse_workbook(response)
        sheet = workbook["#41 - WA 41"]
        headers = [cell.value for cell in sheet[1]]
        self.assertNotIn("Explanatory text question #1", headers)



# Standard Library
from unittest.mock import patch

# Django
from django.test import TestCase

# Project
from django_easy_quiz.models import (
    InterpretationQuiz,
    InterpretationQuizAnswer,
    InterpretationQuizConclusion,
    InterpretationQuizQuestion,
    SavedQuiz,
)


class InterpretationQuizViewTests(TestCase):
    def _create_quiz(self):
        quiz = InterpretationQuiz.objects.create(
            name="Interpretation",
            description="<p>Desc</p>",
        )
        for sym, desc in zip(
            ("■", "▲", "◆", "●"),
            ("<p>C1</p>", "<p>C2</p>", "<p>C3</p>", "<p>C4</p>"),
        ):
            InterpretationQuizConclusion.objects.create(
                quiz=quiz, symbol=sym, description=desc
            )
        q1 = InterpretationQuizQuestion.objects.create(quiz=quiz, label="Q1")
        for order, (sym, lab) in enumerate(
            zip(("■", "▲", "◆", "●"), ("A1", "A2", "A3", "A4")), start=1
        ):
            InterpretationQuizAnswer.objects.create(
                question=q1, symbol=sym, label=lab, order=order
            )
        q2 = InterpretationQuizQuestion.objects.create(quiz=quiz, label="Q2")
        for order, (sym, lab) in enumerate(
            zip(("■", "▲", "◆", "●"), ("B1", "B2", "B3", "B4")), start=1
        ):
            InterpretationQuizAnswer.objects.create(
                question=q2, symbol=sym, label=lab, order=order
            )
        return quiz, q1, q2

    def test_post_majority_symbol_and_saved_quiz(self):
        quiz, q1, q2 = self._create_quiz()
        saved_quiz = SavedQuiz.objects.create(
            quiz_type="Interpretation", quiz_ended=False
        )
        a1 = q1.answers.get(symbol="▲")
        a2 = q2.answers.get(symbol="▲")

        with patch("django_easy_quiz.views.interpretation.SAVE_QUIZZES_RESULTS", True):
            response = self.client.post(
                f"/interpretation/{quiz.id}/",
                data={
                    f"question_{q1.id}": str(a1.id),
                    f"question_{q2.id}": str(a2.id),
                    "saved_quiz": str(saved_quiz.id),
                },
            )

        self.assertEqual(response.status_code, 200)
        saved_quiz.refresh_from_db()
        self.assertTrue(saved_quiz.quiz_ended)
        self.assertEqual(saved_quiz.interpretation_max_symbol, "▲")
        self.assertEqual(saved_quiz.conclusion, "<p>C2</p>")

    def test_post_tie_keeps_first_answer_occurrence(self):
        quiz, q1, q2 = self._create_quiz()
        saved_quiz = SavedQuiz.objects.create(
            quiz_type="Interpretation", quiz_ended=False
        )
        a1_square = q1.answers.get(symbol="■")
        a2_triangle = q2.answers.get(symbol="▲")

        with patch("django_easy_quiz.views.interpretation.SAVE_QUIZZES_RESULTS", True):
            response = self.client.post(
                f"/interpretation/{quiz.id}/",
                data={
                    f"question_{q1.id}": str(a1_square.id),
                    f"question_{q2.id}": str(a2_triangle.id),
                    "saved_quiz": str(saved_quiz.id),
                },
            )

        self.assertEqual(response.status_code, 200)
        saved_quiz.refresh_from_db()
        self.assertEqual(saved_quiz.interpretation_max_symbol, "■")
        self.assertEqual(saved_quiz.conclusion, "<p>C1</p>")



# Django
from django.contrib import admin, messages
from django.http import HttpResponseRedirect
from django.shortcuts import reverse
from django.utils.html import format_html, strip_tags
from django.utils.safestring import mark_safe
from django.utils.timezone import now
from django.utils.translation import gettext as _

# Project
from django_easy_quiz.exporters.saved_quiz_export import xlsx_response
from django_easy_quiz.models import (
    InterpretationQuiz,
    InterpretationQuizAnswer,
    InterpretationQuizConclusion,
    InterpretationQuizQuestion,
    MoreInfoAnswer,
    MoreInfoQuestion,
    MoreInfoQuiz,
    SavedQuiz,
    WeightedAnswersQuiz,
    WeightedAnswersQuizAnswer,
    WeightedAnswersQuizConclusion,
    WeightedAnswersQuizQuestion,
)
from django_easy_quiz.settings import GATHER_STATISTICS
from django_easy_quiz.utils import admin_error_min_max_conclusion


class EasyQuizAdminWideMixin:
    class Media:
        css = {"all": ("django_easy_quiz/css/admin_quiz_wide.css",)}


def nb_of_questions(obj):
    try:
        return str(len(obj.weightedanswersquizquestion_set.all()))
    except AttributeError:
        return str(len(obj.interpretationquizquestion_set.all()))


class WeightedAnswersQuizConclusionAdminInline(admin.StackedInline):
    model = WeightedAnswersQuizConclusion
    extra = 0
    min_num = 1

    fieldsets = (
        (
            None,
            {
                "fields": (
                    ("min_points", "max_points"),
                    "description",
                ),
            },
        ),
    )


class WeightedAnswersQuizQuestionAdminInline(admin.StackedInline):
    model = WeightedAnswersQuizQuestion
    extra = 0
    min_num = 1
    show_change_link = True
    question_number_count = 0
    readonly_fields = ("question_number", "edit_answers_link")
    fieldsets = (
        (
            None,
            {
                "fields": (
                    "question_number",
                    "order",
                    "label",
                    "multiple_answers",
                    "question_explanation_text",
                    "edit_answers_link",
                ),
                "description": _(
                    "Save the quiz, then open the question page to add or edit answers."
                ),
            },
        ),
    )

    @admin.display(description="")
    def question_number(self, obj):
        self.question_number_count += 1
        return _("Question number") + " " + str(self.question_number_count)

    @admin.display(description=_("Answers"))
    def edit_answers_link(self, obj):
        if not obj.pk:
            return _("Save the quiz first.")
        url = reverse(
            "admin:django_easy_quiz_weightedanswersquizquestion_change", args=[obj.pk]
        )
        win = f"waq_answers_{obj.pk}"
        return format_html(
            '<a class="button" href="{url}" '
            "onclick=\"window.open(this.href, '{win}', "
            "'width=1200,height=800,scrollbars=yes,resizable=yes'); "
            'return false;">{label}</a>',
            url=url,
            win=win,
            label=_("Edit answers for this question"),
        )


class WeightedAnswersQuizAnswerAdminInline(admin.StackedInline):
    model = WeightedAnswersQuizAnswer
    extra = 0
    fieldsets = (
        (None, {"fields": ("label",)}),
        (None, {"fields": (("points", "order"),)}),
    )


@admin.register(WeightedAnswersQuizQuestion)
class WeightedAnswersQuizQuestionAdmin(EasyQuizAdminWideMixin, admin.ModelAdmin):
    list_display = ("id", "quiz", "order", "label", "multiple_answers")
    list_filter = ("quiz", "multiple_answers")
    search_fields = ("label", "quiz__name")
    inlines = [WeightedAnswersQuizAnswerAdminInline]


@admin.register(WeightedAnswersQuiz)
class WeightedAnswersQuizAdmin(EasyQuizAdminWideMixin, admin.ModelAdmin):
    if GATHER_STATISTICS:
        fieldsets = (
            (
                None,
                {
                    "fields": (
                        "name",
                        "questions_random",
                        "answers_random",
                        "show_score_summary",
                        "show_answers_corrections_text",
                        "description",
                        "more_info_quiz",
                    ),
                },
            ),
        )
    else:
        fieldsets = (
            (
                None,
                {
                    "fields": (
                        "name",
                        "questions_random",
                        "answers_random",
                        "show_score_summary",
                        "show_answers_corrections_text",
                        "description",
                    ),
                },
            ),
        )

    list_display = (
        "id",
        "name",
        nb_of_questions,
        "questions_random",
        "answers_random",
    )
    list_display_links = (
        "id",
        "name",
    )
    list_filter = (
        "questions_random",
        "answers_random",
        "create_date",
    )
    search_fields = ("name",)

    inlines = [
        WeightedAnswersQuizQuestionAdminInline,
        WeightedAnswersQuizConclusionAdminInline,
    ]

    def save_formset(self, request, form, formset, change):
        """
        Save formset, BUT we will launch some checks before:
            - check that no conclusion have min_points > max_points
            - check that all conclusions create only one (enclosed) number sequence and that the min(answers_points_sum) & max(answers_points_sum) are within its bounds
        """
        instances = formset.save(commit=False)
        for instance in instances:
            instance.save()
        # if checkbox "Delete" is ticked then delete questions
        for obj in formset.deleted_objects:
            obj.delete()

        # question formset is treated before conclusion, so in conclusion formset we can get the quiz, the questions, and the conclusion (using the formset.cleaned_data)
        if "WeightedAnswersQuizConclusionFormFormSet" in str(type(formset)):
            min_points_sum = 0
            max_points_sum = 0
            conclusions_combinations = []

            for question in (
                formset[0].cleaned_data["quiz"].weightedanswersquizquestion_set.all()
            ):
                points = [a.points for a in question.answers.all()]
                if not points:
                    messages.error(
                        request,
                        _('Question "%(q)s" must have at least one answer.')
                        % {"q": question.label},
                        extra_tags="easy_quiz_error",
                    )
                    continue
                min_points_sum += min(points)
                max_points_sum += max(points)

            for conclusion in formset.cleaned_data:
                if "min_points" in conclusion and "max_points" in conclusion:
                    if conclusion["min_points"] > conclusion["max_points"]:
                        # there is a problem, so we create a message with an extra tag using this function:
                        admin_error_min_max_conclusion(
                            request,
                            conclusion["min_points"],
                            conclusion["max_points"],
                            conclusion["description"],
                        )
                    conclusions_combinations.append(
                        [conclusion["min_points"], conclusion["max_points"]]
                    )
        # we need to save formset even if it have errors, because django needs its id somewhere later, and we don't want to make the user have to re-type the answers & conclusions
        formset.save_m2m()

    def response_add(self, request, obj, post_url_continue=None):
        """
        Method to get url to redirect after having created a new quiz:
            - Redirect to list view (using super) if there's no message with extra tag "easy_quiz_error".
            - Redirect to change view if a message with extra tag "easy_quiz_error" exist.
        """
        storage = messages.get_messages(request)
        for message in storage:
            if message.extra_tags == "easy_quiz_error":
                storage.used = False
                return HttpResponseRedirect(
                    reverse(
                        "admin:{}_{}_change".format(
                            obj._meta.app_label, obj._meta.model_name
                        ),
                        args=[obj.id],
                    )
                )
        storage.used = False
        return super().response_change(request, obj)

    def response_change(self, request, obj):
        """
        Method to get url to redirect after having modified a quiz:
            - Redirect to list view (using super) if there's no message with extra tag "easy_quiz_error".
            - Redirect to change view if a message with extra tag "easy_quiz_error" exist.
        """
        storage = messages.get_messages(request)
        for message in storage:
            if message.extra_tags == "easy_quiz_error":
                storage.used = False
                return HttpResponseRedirect(
                    reverse(
                        "admin:{}_{}_change".format(
                            obj._meta.app_label, obj._meta.model_name
                        ),
                        args=[obj.id],
                    )
                )
        storage.used = False

        return super().response_change(request, obj)


# InterpretationQuiz


class InterpretationQuizConclusionAdminInline(admin.TabularInline):
    model = InterpretationQuizConclusion
    extra = 0
    min_num = 1


class InterpretationQuizQuestionAdminInline(admin.StackedInline):
    model = InterpretationQuizQuestion
    extra = 0
    min_num = 1
    show_change_link = True
    question_number_count = 0
    readonly_fields = ("question_number", "edit_answers_link")
    fieldsets = (
        (
            None,
            {
                "fields": (
                    "question_number",
                    "order",
                    "label",
                    "edit_answers_link",
                ),
                "description": _(
                    "Save the quiz, then open the question page to add or edit answers."
                ),
            },
        ),
    )

    @admin.display(description="")
    def question_number(self, obj):
        self.question_number_count += 1
        return _("Question number") + " " + str(self.question_number_count)

    @admin.display(description=_("Answers"))
    def edit_answers_link(self, obj):
        if not obj.pk:
            return _("Save the quiz first.")
        url = reverse(
            "admin:django_easy_quiz_interpretationquizquestion_change", args=[obj.pk]
        )
        win = f"iq_answers_{obj.pk}"
        return format_html(
            '<a class="button" href="{url}" '
            "onclick=\"window.open(this.href, '{win}', "
            "'width=1200,height=800,scrollbars=yes,resizable=yes'); "
            'return false;">{label}</a>',
            url=url,
            win=win,
            label=_("Edit answers for this question"),
        )


class InterpretationQuizAnswerAdminInline(admin.TabularInline):
    model = InterpretationQuizAnswer
    extra = 0


@admin.register(InterpretationQuizQuestion)
class InterpretationQuizQuestionAdmin(EasyQuizAdminWideMixin, admin.ModelAdmin):
    list_display = ("id", "quiz", "order", "label")
    list_filter = ("quiz",)
    search_fields = ("label", "quiz__name")
    inlines = [InterpretationQuizAnswerAdminInline]


@admin.register(InterpretationQuiz)
class InterpretationQuizAdmin(EasyQuizAdminWideMixin, admin.ModelAdmin):
    if GATHER_STATISTICS:
        fieldsets = (
            (
                None,
                {
                    "fields": (
                        "name",
                        "questions_random",
                        "answers_random",
                        "description",
                        "more_info_quiz",
                    ),
                },
            ),
        )
    else:
        fieldsets = (
            (
                None,
                {
                    "fields": (
                        "name",
                        "questions_random",
                        "answers_random",
                        "description",
                    ),
                },
            ),
        )

    list_display = (
        "id",
        "name",
        nb_of_questions,
        "questions_random",
        "answers_random",
    )
    list_display_links = (
        "id",
        "name",
    )
    list_filter = (
        "questions_random",
        "answers_random",
        "create_date",
    )
    search_fields = ("name",)

    inlines = [
        InterpretationQuizQuestionAdminInline,
        InterpretationQuizConclusionAdminInline,
    ]


# activate stats only if activated in settings (default = False)
if GATHER_STATISTICS:

    @admin.register(SavedQuiz)
    class SavedQuizAdmin(EasyQuizAdminWideMixin, admin.ModelAdmin):
        actions = ("export_saved_quiz_xlsx",)

        def has_change_permission(self, request, obj=None):
            return False

        def has_add_permission(self, request, obj=None):
            return False

        readonly_fields = [field.name for field in SavedQuiz._meta.get_fields()]

        list_display = ("quiz_type", "started_quiz", "quiz_ended", "uuid")
        list_filter = (
            "quiz_ended",
            "weighted_answers_quiz",
            "interpretation_quiz",
        )

        fields = (
            "quiz_type",
            "quiz_ended",
            "answers_admin",
            "conclusion_points_admin",
            "conclusion_admin",
            "more_infos_admin",
        )

        @admin.display(description=_("Answers"))
        def answers_admin(self, obj):
            answers_txt = "<ul>"
            for answers in obj.answers:
                answers_txt += f"<li>{answers['question']['label']}<ul>"
                if "answers" in answers:  # weighted answers quiz
                    for answer in answers["answers"]:
                        if isinstance(answer, dict):
                            if answer.get("selected"):
                                answers_txt += (
                                    f"<li>{strip_tags(answer.get('label', ''))}</li>"
                                )
                        else:
                            answers_txt += f"<li>{strip_tags(answer)}</li>"
                    question_explanation_text = answers.get(
                        "question_explanation_text"
                    ) or answers["question"].get("question_explanation_text")
                    if question_explanation_text:
                        answers_txt += (
                            f"<li><i>{strip_tags(question_explanation_text)}</i></li>"
                        )
                else:
                    if "answer" in answers:
                        answers_txt += f"<li>{strip_tags(answers['answer'])}</li>"
                    else:
                        for key in ("answer_1", "answer_2", "answer_3", "answer_4"):
                            if key in answers.get("question", {}):
                                answers_txt += (
                                    f"<li>{strip_tags(answers['question'][key])}</li>"
                                )
                answers_txt += "</ul></li>"
            if answers_txt == "<ul>":
                answers_txt += f"<li>{_('No answer recorded for this quiz.')}</li>"
            answers_txt += "</ul>"
            return mark_safe(answers_txt)

        @admin.display(description=_("Points"))
        def conclusion_points_admin(self, obj):
            if "points" in obj.conclusion:  # weighted_answers quiz
                return obj.conclusion["points"]
            return ""  # interpretation quiz

        @admin.display(description=_("Conclusion"))
        def conclusion_admin(self, obj):
            if "description" in obj.conclusion:  # weighted_answers quiz
                return mark_safe(obj.conclusion["description"])
            return mark_safe(obj.conclusion)  # interpretation quiz

        @admin.display(description=_("More infos"))
        def more_infos_admin(self, obj):
            more_infos_txt = "<ul>"
            for question in obj.more_infos:
                more_infos_txt += f"<li>{question['question']}<ul>"
                more_infos_txt += f"<li>{question['answer']}</li>"
                more_infos_txt += "</ul></li>"
            return mark_safe(more_infos_txt)

        @admin.action(description=_("Export selected saved quizzes as Excel (XLSX)"))
        def export_saved_quiz_xlsx(self, request, queryset):
            saved_quizzes = list(
                queryset.filter(quiz_ended=True).select_related(
                    "weighted_answers_quiz", "interpretation_quiz"
                )
            )
            timestamp = now().strftime("%d-%m-%Y-%H_%M_%S")
            filename = f"{_('Export saved quizzes')} {timestamp}.xlsx"
            return xlsx_response(saved_quizzes, filename)


# activate more info questions only if activated in settings (default = False)
if GATHER_STATISTICS:

    class MoreInfoAnswerAdminInline(admin.StackedInline):
        model = MoreInfoAnswer

    @admin.register(MoreInfoQuestion)
    class MoreInfoQuestionAdmin(EasyQuizAdminWideMixin, admin.ModelAdmin):
        inlines = [
            MoreInfoAnswerAdminInline,
        ]

        readonly_fields = ("quiz",)

    class MoreInfoQuestionAdminInline(admin.StackedInline):
        model = MoreInfoQuestion
        show_change_link = True

    @admin.register(MoreInfoQuiz)
    class MoreInfoQuizAdmin(EasyQuizAdminWideMixin, admin.ModelAdmin):
        fieldsets = (
            (
                None,
                {
                    "fields": (("name"),),
                },
            ),
            (
                None,
                {
                    "fields": [],
                    "description": _(
                        "Save your questions first, and then you will be able to update each question to add answers."
                    ),
                },
            ),
        )

        inlines = [
            MoreInfoQuestionAdminInline,
        ]



# Standard Library
from uuid import uuid4

# Django
from django.db import models
from django.urls import reverse
from django.utils.translation import gettext
from django.utils.translation import gettext_lazy as _

# Third party
from ckeditor_uploader.fields import RichTextUploadingField

# Quiz


class Quiz(models.Model):
    name = models.CharField(_("Quiz name"), max_length=1024)
    questions_random = models.BooleanField(
        default=False,
        verbose_name=_("Random questions"),
        help_text=_("Questions will be shown on random order."),
    )
    answers_random = models.BooleanField(
        default=False,
        verbose_name=_("Random answers"),
        help_text=_("Answers will be shown on random order."),
    )
    create_date = models.DateTimeField(
        auto_now_add=True,
        verbose_name=_("Creation date"),
    )
    description = RichTextUploadingField(verbose_name=_("Description"))
    more_info_quiz = models.ForeignKey(
        "MoreInfoQuiz",
        default=None,
        blank=True,
        null=True,
        on_delete=models.PROTECT,
        verbose_name=_("More info quiz"),
        help_text=_("Will display this 'more info quiz' at the end of the main quiz."),
    )

    def __str__(self):
        return f"{self.name}"

    class Meta:
        abstract = True


class WeightedAnswersQuiz(Quiz):
    show_score_summary = models.BooleanField(
        default=True, verbose_name=_("Show score in summary")
    )
    show_answers_corrections_text = models.BooleanField(
        default=False, verbose_name=_("Show answers explanatory text in summary")
    )

    class Meta:
        verbose_name = _("Weighted Answers Quiz")
        verbose_name_plural = _("Weighted Answers Quizzes")

    def get_absolute_url(self):
        return reverse("weighted_answers_quiz", kwargs={"pk": self.pk})


class InterpretationQuiz(Quiz):
    class Meta:
        verbose_name = _("Interpretation Quiz")
        verbose_name_plural = _("Interpretation Quizzes")

    def get_absolute_url(self):
        return reverse("interpretation_quiz", kwargs={"pk": self.pk})


# Questions


class Question(models.Model):
    label = models.CharField(_("Question"), max_length=1024)

    class Meta:
        abstract = True


class WeightedAnswersQuizQuestion(Question):
    quiz = models.ForeignKey(
        WeightedAnswersQuiz,
        on_delete=models.CASCADE,
        verbose_name=_("Weighted answers quiz"),
    )
    label = models.CharField(verbose_name=_("Question"), max_length=1024)
    order = models.PositiveIntegerField(default=0, verbose_name=_("Display order"))
    multiple_answers = models.BooleanField(
        default=False, verbose_name=_("More than one answer possible")
    )
    question_explanation_text = RichTextUploadingField(
        verbose_name=_("Explanation text for question"), null=True, blank=True
    )

    def __str__(self):
        return f"{self.label}"

    def toJson(self):
        return {
            "label": self.label,
            "multiple_answers": self.multiple_answers,
            "question_explanation_text": self.question_explanation_text,
        }

    class Meta:
        verbose_name = _("Weighted Answers Quiz Question")
        verbose_name_plural = _("Weighted Answers Quiz Questions")
        ordering = ("order", "id")


class InterpretationQuizQuestion(Question):
    quiz = models.ForeignKey(
        InterpretationQuiz,
        on_delete=models.CASCADE,
        verbose_name=_("Interpretation quiz"),
    )
    label = models.CharField(verbose_name=_("Question"), max_length=1024)
    order = models.PositiveIntegerField(default=0, verbose_name=_("Display order"))

    def __str__(self):
        return f"{self.label}"

    def toJson(self):
        answers_by_symbol = {
            answer.symbol: answer.label
            for answer in self.answers.all().order_by("order", "id")
        }
        return {
            "label": self.label,
            "answer_1": f"■ - {answers_by_symbol.get('■', '')}",
            "answer_2": f"▲ - {answers_by_symbol.get('▲', '')}",
            "answer_3": f"◆ - {answers_by_symbol.get('◆', '')}",
            "answer_4": f"● - {answers_by_symbol.get('●', '')}",
        }

    class Meta:
        verbose_name = _("Interpretation Quiz Question")
        verbose_name_plural = _("Interpretation Quiz Questions")
        ordering = ("order", "id")


# Answers


class Answer(models.Model):
    answer = RichTextUploadingField(verbose_name=_("Answer"))

    def __str__(self):
        return gettext("Answer #%s") % self.id

    class Meta:
        abstract = True


INTERPRETATION_QUESTION_CHOICES = (
    ("■", "■"),
    ("▲", "▲"),
    ("◆", "◆"),
    ("●", "●"),
)


class WeightedAnswersQuizAnswer(models.Model):
    question = models.ForeignKey(
        WeightedAnswersQuizQuestion,
        on_delete=models.CASCADE,
        related_name="answers",
        verbose_name=_("Question"),
    )
    label = RichTextUploadingField(verbose_name=_("Answer"))
    points = models.IntegerField(verbose_name=_("Points for answer"))
    order = models.PositiveIntegerField(default=0, verbose_name=_("Display order"))

    def __str__(self):
        return ""

    class Meta:
        verbose_name = _("Weighted Answers Quiz Answer")
        verbose_name_plural = _("Weighted Answers Quiz Answers")
        ordering = ("order", "id")


class InterpretationQuizAnswer(models.Model):
    question = models.ForeignKey(
        InterpretationQuizQuestion,
        on_delete=models.CASCADE,
        related_name="answers",
        verbose_name=_("Question"),
    )
    symbol = models.CharField(
        max_length=1,
        choices=INTERPRETATION_QUESTION_CHOICES,
        verbose_name=_("Answer symbol"),
    )
    label = RichTextUploadingField(verbose_name=_("Answer"))
    order = models.PositiveIntegerField(default=0, verbose_name=_("Display order"))

    class Meta:
        verbose_name = _("Interpretation Quiz Answer")
        verbose_name_plural = _("Interpretation Quiz Answers")
        ordering = ("order", "id")


# Conclusions/results


class QuizConclusion(models.Model):
    description = RichTextUploadingField(verbose_name=_("Description"))

    class Meta:
        abstract = True


class WeightedAnswersQuizConclusion(QuizConclusion):
    quiz = models.ForeignKey(
        WeightedAnswersQuiz,
        on_delete=models.CASCADE,
        verbose_name=_("Weighted answers quiz"),
    )
    min_points = models.IntegerField(verbose_name=_("Min amount of points (included)"))
    max_points = models.IntegerField(verbose_name=_("Max amount of points (included)"))

    def __str__(self):
        return gettext("Weighted Answers Quiz Conclusion") + " #" + str(self.id)

    class Meta:
        verbose_name = _("Weighted Answers Quiz Conclusion")
        verbose_name_plural = _("Weighted Answers Quiz Conclusions")


class InterpretationQuizConclusion(models.Model):
    quiz = models.ForeignKey(
        InterpretationQuiz,
        on_delete=models.CASCADE,
        related_name="conclusions",
        verbose_name=_("Interpretation quiz"),
    )
    symbol = models.CharField(
        max_length=1,
        choices=INTERPRETATION_QUESTION_CHOICES,
        verbose_name=_("Conclusion symbol"),
    )
    description = RichTextUploadingField(verbose_name=_("Description"))

    class Meta:
        verbose_name = _("Interpretation Quiz Conclusion")
        verbose_name_plural = _("Interpretation Quiz Conclusions")
        constraints = [
            models.UniqueConstraint(
                fields=("quiz", "symbol"),
                name="uniq_interpretation_conclusion_symbol_per_quiz",
            )
        ]


# Djangocms Plugins


class BasePlugin(models.Model):
    LEVEL_1 = "h1"
    LEVEL_2 = "h2"
    LEVEL_3 = "h3"
    LEVEL_4 = "h4"
    LEVEL_5 = "h5"
    LEVEL_6 = "h6"
    TITLE_LEVEL = [
        (LEVEL_1, _("Title level 1 (<h1>)")),
        (LEVEL_2, _("Title level 2 (<h2>)")),
        (LEVEL_3, _("Title level 3 (<h3>)")),
        (LEVEL_4, _("Title level 4 (<h4>)")),
        (LEVEL_5, _("Title level 5 (<h5>)")),
        (LEVEL_6, _("Title level 6 (<h6>)")),
    ]

    title_level = models.CharField(
        verbose_name=_("Title level"),
        choices=TITLE_LEVEL,
        max_length=4,
        help_text=_("Heading level used to render the quiz title."),
    )
    short_description = models.CharField(
        _("Short description"),
        max_length=1024,
        null=True,
        blank=True,
    )
    button_text = models.CharField(
        _("Button text"),
        max_length=1024,
        null=True,
        blank=True,
    )

    class Meta:
        abstract = True


class SavedQuiz(models.Model):
    uuid = models.UUIDField(default=uuid4, editable=False, verbose_name=_("UUID"))
    quiz_type = models.CharField(max_length=255, verbose_name=_("Quiz type"))
    started_quiz = models.DateTimeField(
        auto_now_add=True, verbose_name=_("Started quiz")
    )
    quiz_ended = models.BooleanField(default=False, verbose_name=_("Quiz ended"))
    answers = models.JSONField(default=dict, verbose_name=_("Answers"))
    weighted_answers_quiz = models.ForeignKey(
        WeightedAnswersQuiz,
        null=True,
        blank=True,
        on_delete=models.PROTECT,
        verbose_name=_("Weighted answers quiz"),
    )
    interpretation_quiz = models.ForeignKey(
        InterpretationQuiz,
        null=True,
        blank=True,
        on_delete=models.PROTECT,
        verbose_name=_("Interpretation quiz"),
    )
    interpretation_max_symbol = models.CharField(
        max_length=32,
        null=True,
        blank=True,
        verbose_name=_("Interpretation max symbol"),
    )
    conclusion = models.JSONField(default=dict, verbose_name=_("Conclusion"))
    more_infos = models.JSONField(default=dict, verbose_name=_("More infos"))

    class Meta:
        verbose_name = _("Saved Quiz")
        verbose_name_plural = _("Saved Quizzes")


class MoreInfoQuiz(models.Model):
    name = models.CharField(
        max_length=255,
        help_text=_("Will be displayed below the results."),
        verbose_name=_("Name"),
    )

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _("More-Info Quiz")
        verbose_name_plural = _("More-Info Quizzes")


class MoreInfoQuestion(models.Model):
    label = models.CharField(max_length=255, verbose_name=_("Label"))
    quiz = models.ForeignKey(
        MoreInfoQuiz, on_delete=models.PROTECT, verbose_name=_("Quiz")
    )
    is_required = models.BooleanField(
        default=False,
        help_text=_("Is this question required?"),
        verbose_name=_("Is required"),
    )
    answer_is_txt = models.BooleanField(
        default=False,
        help_text=_(
            "Check this parameter to allow plain text answers instead of pre-defined answers."
        ),
        verbose_name=_("Answer is text"),
    )
    answer_is_list = models.BooleanField(
        default=False,
        help_text=_(
            "Check this parameter to display a big list of answers using a select input and not radio buttons."
        ),
        verbose_name=_("Answer is list"),
    )

    def __str__(self):
        return self.label

    class Meta:
        verbose_name = _("More-Info Question")
        verbose_name_plural = _("More-Info Questions")


class MoreInfoAnswer(models.Model):
    question = models.ForeignKey(
        MoreInfoQuestion, on_delete=models.PROTECT, verbose_name=_("Question")
    )
    answer = models.CharField(max_length=512, verbose_name=_("Answer"))

    def __str__(self):
        return self.answer

    class Meta:
        verbose_name = _("More-Info Answer")
        verbose_name_plural = _("More-Info Answers")


try:
    # Third party
    from cms.models.pluginmodel import CMSPlugin

    class WeightedAnswersPlugin(BasePlugin, CMSPlugin):
        quiz = models.ForeignKey(
            WeightedAnswersQuiz,
            on_delete=models.SET_NULL,
            null=True,
            verbose_name=_("Weighted answers quiz"),
        )

        class Meta:
            verbose_name = _("Weighted Answers Quiz")
            verbose_name_plural = _("Weighted Answers Quizzes")

    class InterpretationPlugin(BasePlugin, CMSPlugin):
        quiz = models.ForeignKey(
            InterpretationQuiz,
            on_delete=models.SET_NULL,
            null=True,
            verbose_name=_("Interpretation quiz"),
        )

        class Meta:
            verbose_name = _("Interpretation Quiz")
            verbose_name_plural = _("Interpretation Quizzes")

except ImportError:
    ...



# Third party
from hypothesis import given
from hypothesis import strategies as st

# Project
from django_easy_quiz.exporters.saved_quiz_export import (
    _question_answer,
    _question_explanation,
    _sanitize_sheet_title_text,
    _sheet_title,
    _to_text,
)


@given(st.text())
def test_to_text_always_returns_string(value):
    assert isinstance(_to_text(value), str)


@given(st.text())
def test_sanitize_sheet_title_text_never_empty(value):
    sanitized = _sanitize_sheet_title_text(value)
    assert isinstance(sanitized, str)
    assert sanitized != ""


@given(st.integers(min_value=1, max_value=999999), st.text())
def test_sheet_title_respects_excel_constraints(quiz_id, display_name):
    title = _sheet_title(quiz_id, display_name)
    assert isinstance(title, str)
    assert 1 <= len(title) <= 31
    assert title.startswith(f"#{quiz_id} - ")


@given(
    st.lists(
        st.one_of(
            st.none(),
            st.text(),
            st.fixed_dictionaries(
                {
                    "label": st.text(),
                    "info": st.text(),
                }
            ),
        ),
        max_size=10,
    )
)
def test_question_answer_helpers_handle_legacy_and_new_formats(answers):
    payload = {"answers": answers}
    answer = _question_answer(payload)
    question_explanation = _question_explanation(payload)
    assert isinstance(answer, str)
    assert isinstance(question_explanation, str)



"""Structured export helpers for SavedQuiz admin actions."""

# Standard Library
import re
import unicodedata
from html import unescape

# Django
from django.http import HttpResponse
from django.utils.html import strip_tags
from django.utils.text import slugify
from django.utils.translation import gettext as _

# Third party
from openpyxl import Workbook
from openpyxl.styles import Font

BASE_COLUMNS = [
    "id",
    "uuid",
    "started_quiz",
    "quiz_type",
    "conclusion_points",
    "conclusion_text",
]
MAX_COLUMN_WIDTH = 85.0  # ~= 600px in Excel units
MAX_SHEET_TITLE_LENGTH = 31


def _safe_slug(value):
    slug = slugify(str(value or ""))
    return slug or "unknown"


def _to_text(value):
    if value is None:
        return ""
    cleaned = strip_tags(str(value))
    # Decode HTML entities coming from rich text fields (&eacute;, &nbsp;, ...)
    return unescape(cleaned).replace("\xa0", " ")


def _question_label(answer_entry):
    question_payload = answer_entry.get("question", {})
    if isinstance(question_payload, dict):
        return _to_text(question_payload.get("label", ""))
    return _to_text(question_payload)


def _question_answer(answer_entry):
    selected_answers = answer_entry.get("answers")
    if isinstance(selected_answers, list):
        labels = []
        for item in selected_answers:
            if isinstance(item, dict):
                if item.get("selected"):
                    labels.append(_to_text(item.get("label", "")))
            elif item is not None:
                labels.append(_to_text(item))
        return " | ".join(label for label in labels if label)
    return _to_text(answer_entry.get("answer", ""))


def _question_explanation(answer_entry):
    if "question_explanation_text" in answer_entry:
        return _to_text(answer_entry.get("question_explanation_text", ""))
    question_payload = answer_entry.get("question", {})
    if isinstance(question_payload, dict):
        return _to_text(question_payload.get("question_explanation_text", ""))
    return ""


def _flatten_saved_quiz(saved_quiz):
    row = {
        "id": saved_quiz.id,
        "uuid": str(saved_quiz.uuid),
        "started_quiz": saved_quiz.started_quiz.isoformat()
        if saved_quiz.started_quiz
        else "",
        "quiz_type": _to_text(saved_quiz.quiz_type),
        "conclusion_points": "",
        "conclusion_text": "",
    }

    conclusion = (
        saved_quiz.conclusion if isinstance(saved_quiz.conclusion, dict) else {}
    )
    row["conclusion_points"] = conclusion.get("points", "")
    if "description" in conclusion:
        row["conclusion_text"] = _to_text(conclusion["description"])
    elif saved_quiz.conclusion:
        row["conclusion_text"] = _to_text(saved_quiz.conclusion)

    for info in saved_quiz.more_infos or []:
        info_key = f"info__{_safe_slug(info.get('question', 'question'))}"
        row[info_key] = _to_text(info.get("answer", ""))

    questions = [
        {
            "index": index,
            "label": _question_label(answer_entry),
            "answer": _question_answer(answer_entry),
            "question_explanation": _question_explanation(answer_entry),
        }
        for index, answer_entry in enumerate(saved_quiz.answers or [], start=1)
    ]
    return row, questions


def _base_label(column_key):
    labels = {
        "id": _("ID"),
        "uuid": _("UUID"),
        "started_quiz": _("Start date"),
        "quiz_type": _("Quiz name"),
        "conclusion_points": _("Points earned"),
        "conclusion_text": _("Conclusion obtained"),
    }
    return labels.get(column_key, column_key)


def _info_header_label(info_key, info_label_by_key):
    """Use human-readable question text; slug is only an internal column key."""
    return info_label_by_key.get(info_key) or info_key.replace("info__", "")


def _question_column_label(index, question_text):
    return _("Question #%(index)s: %(question)s") % {
        "index": index,
        "question": question_text or _("No label"),
    }


def _answer_column_label(index):
    return _("Answer question #%(index)s") % {"index": index}


def _question_explanation_column_label(index):
    return _("Explanatory text question #%(index)s") % {"index": index}


def _quiz_group_key(saved_quiz):
    if saved_quiz.weighted_answers_quiz_id:
        quiz_obj = saved_quiz.weighted_answers_quiz
        return (
            f"wa_{saved_quiz.weighted_answers_quiz_id}",
            saved_quiz.weighted_answers_quiz_id,
            quiz_obj.name if quiz_obj else str(saved_quiz.quiz_type),
            getattr(quiz_obj, "create_date", None),
            bool(getattr(quiz_obj, "show_answers_corrections_text", False)),
        )
    if saved_quiz.interpretation_quiz_id:
        quiz_obj = saved_quiz.interpretation_quiz
        return (
            f"int_{saved_quiz.interpretation_quiz_id}",
            saved_quiz.interpretation_quiz_id,
            quiz_obj.name if quiz_obj else str(saved_quiz.quiz_type),
            getattr(quiz_obj, "create_date", None),
            False,
        )
    return (
        f"unknown_{saved_quiz.id}",
        saved_quiz.id,
        str(saved_quiz.quiz_type),
        None,
        False,
    )


def _group_saved_quizzes(saved_quizzes):
    groups = {}
    for saved_quiz in saved_quizzes:
        key, quiz_id, display_name, created_at, show_explanations = _quiz_group_key(
            saved_quiz
        )
        groups.setdefault(
            key,
            {
                "quiz_id": quiz_id,
                "display_name": display_name,
                "created_at": created_at,
                "show_explanations": show_explanations,
                "items": [],
            },
        )
        groups[key]["items"].append(saved_quiz)

    def _sort_key(item):
        created_at = item[1]["created_at"]
        return created_at or ""

    return sorted(groups.items(), key=_sort_key, reverse=True)


def _build_group_table(saved_quizzes, include_question_explanations=False):
    normalized_rows = []
    info_headers = []
    info_seen = set()
    info_label_by_key = {}
    question_labels_by_index = {}
    max_question_index = 0

    for saved_quiz in saved_quizzes:
        row, questions = _flatten_saved_quiz(saved_quiz)
        normalized_rows.append((row, questions))

        for info in saved_quiz.more_infos or []:
            info_key = f"info__{_safe_slug(info.get('question', 'question'))}"
            label = _to_text(info.get("question", ""))
            if info_key not in info_label_by_key and label:
                info_label_by_key[info_key] = label

        for key in row.keys():
            if key.startswith("info__") and key not in info_seen:
                info_seen.add(key)
                info_headers.append(key)

        for question in questions:
            max_question_index = max(max_question_index, question["index"])
            if question["index"] not in question_labels_by_index and question["label"]:
                question_labels_by_index[question["index"]] = question["label"]

    headers = []
    headers.extend([(key, _base_label(key)) for key in BASE_COLUMNS])
    headers.extend(
        [(key, _info_header_label(key, info_label_by_key)) for key in info_headers]
    )
    for index in range(1, max_question_index + 1):
        question_text = question_labels_by_index.get(index, "")
        headers.append(
            (f"q{index:02d}__question", _question_column_label(index, question_text))
        )
        headers.append((f"q{index:02d}__answer", _answer_column_label(index)))
        if include_question_explanations:
            headers.append(
                (
                    f"q{index:02d}__question_explanation",
                    _question_explanation_column_label(index),
                )
            )

    rows = []
    for row_data, questions in normalized_rows:
        flat_row = {key: row_data.get(key, "") for key, _ in headers}
        for question in questions:
            flat_row[f"q{question['index']:02d}__question"] = question["label"]
            flat_row[f"q{question['index']:02d}__answer"] = question["answer"]
            if include_question_explanations:
                flat_row[f"q{question['index']:02d}__question_explanation"] = question[
                    "question_explanation"
                ]
        rows.append(flat_row)

    return headers, rows


def _sanitize_sheet_title_text(value):
    normalized = unicodedata.normalize("NFKD", str(value or ""))
    ascii_only = normalized.encode("ascii", "ignore").decode("ascii")
    cleaned = re.sub(r"[^A-Za-z0-9 _-]+", "", ascii_only)
    cleaned = re.sub(r"\s+", " ", cleaned).strip()
    return cleaned or "quiz"


def _sheet_title(quiz_id, display_name):
    prefix = f"#{quiz_id} - "
    safe_name = _sanitize_sheet_title_text(display_name)
    max_name_length = max(1, MAX_SHEET_TITLE_LENGTH - len(prefix))
    return f"{prefix}{safe_name[:max_name_length]}".strip()


def _apply_header_style_and_width(worksheet):
    for cell in worksheet[1]:
        cell.font = Font(name="Arial", bold=True)

    for column in worksheet.columns:
        letter = column[0].column_letter
        max_length = 0
        for cell in column:
            max_length = max(max_length, len(str(cell.value or "")))
        worksheet.column_dimensions[letter].width = min(
            MAX_COLUMN_WIDTH, max_length + 2
        )


def xlsx_response(saved_quizzes, filename):
    workbook = Workbook()
    default_sheet = workbook.active
    workbook.remove(default_sheet)

    ended_quizzes = [
        saved_quiz for saved_quiz in saved_quizzes if saved_quiz.quiz_ended
    ]
    grouped = _group_saved_quizzes(ended_quizzes)

    for group_key, group_data in grouped:
        group_items = [item for item in group_data["items"] if item.answers]
        if not group_items:
            continue

        headers, rows = _build_group_table(
            group_items,
            include_question_explanations=group_data["show_explanations"],
        )
        sheet_title = _sheet_title(group_data["quiz_id"], group_data["display_name"])
        worksheet = workbook.create_sheet(title=sheet_title)
        worksheet.append([label for _, label in headers])
        for row in rows:
            worksheet.append([row.get(key, "") for key, _ in headers])
        _apply_header_style_and_width(worksheet)

    if not workbook.sheetnames:
        worksheet = workbook.create_sheet(title="saved_quizzes")
        fallback_headers = [_base_label(key) for key in BASE_COLUMNS]
        worksheet.append(fallback_headers)
        _apply_header_style_and_width(worksheet)

    response = HttpResponse(
        content_type=(
            "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
        )
    )
    response["Content-Disposition"] = f'attachment; filename="{filename}"'
    workbook.save(response)
    return response



# Generated by Django 3.2.24 on 2026-03-31 12:00

# Django
from django.db import migrations, models

# Third party
import ckeditor_uploader.fields


class Migration(migrations.Migration):

    dependencies = [
        ("django_easy_quiz", "0009_moreinfoquestion_is_required"),
    ]

    operations = [
        migrations.AddField(
            model_name="weightedanswersquiz",
            name="show_answers_corrections_text",
            field=models.BooleanField(
                default=False, verbose_name="Show answers explanatory text in summary"
            ),
        ),
        migrations.AddField(
            model_name="weightedanswersquizquestion",
            name="question_explanation_text",
            field=ckeditor_uploader.fields.RichTextUploadingField(
                blank=True, null=True, verbose_name="Explanation text for question"
            ),
        ),
    ]



# Standard Library
from types import SimpleNamespace

# Django
from django.contrib.admin.sites import AdminSite
from django.contrib.messages.storage.fallback import FallbackStorage
from django.test import RequestFactory, TestCase

# Project
from django_easy_quiz.admin import WeightedAnswersQuizAdmin, nb_of_questions
from django_easy_quiz.forms import InterpretationQuizForm, WeightedAnswersQuizForm
from django_easy_quiz.models import (
    InterpretationQuiz,
    InterpretationQuizAnswer,
    InterpretationQuizConclusion,
    InterpretationQuizQuestion,
    MoreInfoAnswer,
    MoreInfoQuestion,
    MoreInfoQuiz,
    WeightedAnswersQuiz,
    WeightedAnswersQuizAnswer,
    WeightedAnswersQuizQuestion,
)
from django_easy_quiz.utils import get_more_info_quiz_formset, get_quiz_and_formset


class UtilsTests(TestCase):
    def test_get_quiz_and_formset_weighted_filters_empty_answers(self):
        quiz = WeightedAnswersQuiz.objects.create(name="W", description="D")
        question = WeightedAnswersQuizQuestion.objects.create(quiz=quiz, label="Q")
        WeightedAnswersQuizAnswer.objects.create(
            question=question, label="A1", points=1, order=1
        )
        _, formset = get_quiz_and_formset(
            WeightedAnswersQuiz, WeightedAnswersQuizForm, quiz.id
        )
        answers = formset.forms[0].initial["answers"]
        self.assertEqual(len(answers), 1)
        self.assertEqual(answers[0]["label"], "A1")

    def test_get_quiz_and_formset_interpretation_symbols(self):
        quiz = InterpretationQuiz.objects.create(name="I", description="D")
        for sym, desc in zip(
            ("■", "▲", "◆", "●"),
            ("c1", "c2", "c3", "c4"),
        ):
            InterpretationQuizConclusion.objects.create(
                quiz=quiz, symbol=sym, description=desc
            )
        iq = InterpretationQuizQuestion.objects.create(quiz=quiz, label="QI")
        InterpretationQuizAnswer.objects.create(
            question=iq, symbol="■", label="A1", order=1
        )
        InterpretationQuizAnswer.objects.create(
            question=iq, symbol="▲", label="A2", order=2
        )
        InterpretationQuizAnswer.objects.create(
            question=iq, symbol="●", label="A4", order=4
        )
        _, formset = get_quiz_and_formset(
            InterpretationQuiz, InterpretationQuizForm, quiz.id
        )
        answers = formset.forms[0].initial["answers"]
        self.assertEqual([a["symbol"] for a in answers], ["■", "▲", "●"])

    def test_get_more_info_quiz_formset_builds_answers(self):
        more_info_quiz = MoreInfoQuiz.objects.create(name="Stats")
        question = MoreInfoQuestion.objects.create(label="Age", quiz=more_info_quiz)
        answer = MoreInfoAnswer.objects.create(question=question, answer="18-25")
        formset = get_more_info_quiz_formset(more_info_quiz)
        self.assertEqual(formset.forms[0].initial["answers"][0]["id"], answer.id)


class AdminTests(TestCase):
    def test_nb_of_questions_works_for_both_quiz_types(self):
        weighted_obj = SimpleNamespace(
            weightedanswersquizquestion_set=SimpleNamespace(all=lambda: [1, 2, 3])
        )
        interpretation_obj = SimpleNamespace(
            interpretationquizquestion_set=SimpleNamespace(all=lambda: [1, 2])
        )
        self.assertEqual(nb_of_questions(weighted_obj), "3")
        self.assertEqual(nb_of_questions(interpretation_obj), "2")

    def test_weighted_admin_response_change_without_errors(self):
        quiz = WeightedAnswersQuiz.objects.create(name="W2", description="D2")
        request = RequestFactory().post("/")
        request.user = SimpleNamespace(
            is_active=True,
            is_staff=True,
            is_superuser=True,
            has_perm=lambda perm: True,
        )
        request.session = {}
        request._messages = FallbackStorage(request)
        admin_obj = WeightedAnswersQuizAdmin(WeightedAnswersQuiz, AdminSite())
        response = admin_obj.response_change(request, quiz)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, "/admin/django_easy_quiz/weightedanswersquiz/")



# Generated by Django 3.2.22 on 2026-03-31 14:19

# Django
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("django_easy_quiz", "0011_auto_20260331_1156"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="interpretationquiz", name="conclusion_max_1"
        ),
        migrations.RemoveField(
            model_name="interpretationquiz", name="conclusion_max_2"
        ),
        migrations.RemoveField(
            model_name="interpretationquiz", name="conclusion_max_3"
        ),
        migrations.RemoveField(
            model_name="interpretationquiz", name="conclusion_max_4"
        ),
        migrations.RemoveField(
            model_name="interpretationquizquestion", name="answer_1"
        ),
        migrations.RemoveField(
            model_name="interpretationquizquestion", name="answer_2"
        ),
        migrations.RemoveField(
            model_name="interpretationquizquestion", name="answer_3"
        ),
        migrations.RemoveField(
            model_name="interpretationquizquestion", name="answer_4"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="answer_1"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="answer_10"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="answer_2"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="answer_3"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="answer_4"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="answer_5"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="answer_6"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="answer_7"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="answer_8"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="answer_9"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="points_answer_1"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="points_answer_10"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="points_answer_2"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="points_answer_3"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="points_answer_4"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="points_answer_5"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="points_answer_6"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="points_answer_7"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="points_answer_8"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="points_answer_9"
        ),
    ]



# Generated by Django 3.2.22 on 2026-03-31 09:56

# Django
import django.db.models.deletion
from django.db import migrations, models

# Third party
import ckeditor_uploader.fields


def copy_denormalized_data(apps, schema_editor):
    WeightedQuestion = apps.get_model("django_easy_quiz", "WeightedAnswersQuizQuestion")
    WeightedAnswer = apps.get_model("django_easy_quiz", "WeightedAnswersQuizAnswer")
    InterpretationQuestion = apps.get_model(
        "django_easy_quiz", "InterpretationQuizQuestion"
    )
    InterpretationAnswer = apps.get_model(
        "django_easy_quiz", "InterpretationQuizAnswer"
    )
    InterpretationQuiz = apps.get_model("django_easy_quiz", "InterpretationQuiz")
    InterpretationConclusion = apps.get_model(
        "django_easy_quiz", "InterpretationQuizConclusion"
    )

    weighted_questions = WeightedQuestion.objects.order_by("quiz_id", "id")
    current_quiz_id = None
    quiz_order = 0
    for question in weighted_questions:
        if question.quiz_id != current_quiz_id:
            current_quiz_id = question.quiz_id
            quiz_order = 1
        question.order = quiz_order
        question.save(update_fields=["order"])
        quiz_order += 1

        for index in range(1, 11):
            label = getattr(question, f"answer_{index}", None)
            points = getattr(question, f"points_answer_{index}", None)
            if label in (None, "") or points is None:
                continue
            WeightedAnswer.objects.create(
                question_id=question.id,
                label=label,
                points=points,
                order=index,
            )

    interpretation_questions = InterpretationQuestion.objects.order_by("quiz_id", "id")
    current_quiz_id = None
    quiz_order = 0
    for question in interpretation_questions:
        if question.quiz_id != current_quiz_id:
            current_quiz_id = question.quiz_id
            quiz_order = 1
        question.order = quiz_order
        question.save(update_fields=["order"])
        quiz_order += 1

        for index, symbol in enumerate(("■", "▲", "◆", "●"), start=1):
            label = getattr(question, f"answer_{index}", None)
            if label in (None, ""):
                continue
            InterpretationAnswer.objects.create(
                question_id=question.id,
                symbol=symbol,
                label=label,
                order=index,
            )

    for quiz in InterpretationQuiz.objects.all():
        for index, symbol in enumerate(("■", "▲", "◆", "●"), start=1):
            description = getattr(quiz, f"conclusion_max_{index}", None)
            if description in (None, ""):
                continue
            InterpretationConclusion.objects.create(
                quiz_id=quiz.id,
                symbol=symbol,
                description=description,
            )


class Migration(migrations.Migration):
    dependencies = [
        ("django_easy_quiz", "0010_weightedanswers_question_explanation"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="interpretationquizquestion",
            options={
                "ordering": ("order", "id"),
                "verbose_name": "Question de quiz d'interprétation",
                "verbose_name_plural": "Questions de quiz d'interprétation",
            },
        ),
        migrations.AlterModelOptions(
            name="weightedanswersquizquestion",
            options={
                "ordering": ("order", "id"),
                "verbose_name": "Question de quiz avec réponses pondérées",
                "verbose_name_plural": "Questions de quiz avec réponses pondérées",
            },
        ),
        migrations.AddField(
            model_name="interpretationquizquestion",
            name="order",
            field=models.PositiveIntegerField(default=0, verbose_name="Display order"),
        ),
        migrations.AddField(
            model_name="weightedanswersquizquestion",
            name="order",
            field=models.PositiveIntegerField(default=0, verbose_name="Display order"),
        ),
        migrations.AlterField(
            model_name="moreinfoquestion",
            name="is_required",
            field=models.BooleanField(
                default=False, help_text="Est-ce que la réponse est requise ?"
            ),
        ),
        migrations.AlterField(
            model_name="weightedanswersquizquestion",
            name="multiple_answers",
            field=models.BooleanField(
                default=False, verbose_name="Plusieurs réponses autorisées"
            ),
        ),
        migrations.CreateModel(
            name="WeightedAnswersQuizAnswer",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "label",
                    ckeditor_uploader.fields.RichTextUploadingField(
                        verbose_name="Answer"
                    ),
                ),
                ("points", models.IntegerField(verbose_name="Points for answer")),
                (
                    "order",
                    models.PositiveIntegerField(
                        default=0, verbose_name="Display order"
                    ),
                ),
                (
                    "question",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="answers",
                        to="django_easy_quiz.weightedanswersquizquestion",
                    ),
                ),
            ],
            options={
                "verbose_name": "Weighted Answers Quiz Answer",
                "verbose_name_plural": "Weighted Answers Quiz Answers",
                "ordering": ("order", "id"),
            },
        ),
        migrations.CreateModel(
            name="InterpretationQuizConclusion",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("description", ckeditor_uploader.fields.RichTextUploadingField()),
                (
                    "symbol",
                    models.CharField(
                        choices=[("■", "■"), ("▲", "▲"), ("◆", "◆"), ("●", "●")],
                        max_length=1,
                        verbose_name="Conclusion symbol",
                    ),
                ),
                (
                    "quiz",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="conclusions",
                        to="django_easy_quiz.interpretationquiz",
                    ),
                ),
            ],
            options={
                "verbose_name": "Interpretation Quiz Conclusion",
                "verbose_name_plural": "Interpretation Quiz Conclusions",
            },
        ),
        migrations.CreateModel(
            name="InterpretationQuizAnswer",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "symbol",
                    models.CharField(
                        choices=[("■", "■"), ("▲", "▲"), ("◆", "◆"), ("●", "●")],
                        max_length=1,
                        verbose_name="Answer symbol",
                    ),
                ),
                (
                    "label",
                    ckeditor_uploader.fields.RichTextUploadingField(
                        verbose_name="Answer"
                    ),
                ),
                (
                    "order",
                    models.PositiveIntegerField(
                        default=0, verbose_name="Display order"
                    ),
                ),
                (
                    "question",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="answers",
                        to="django_easy_quiz.interpretationquizquestion",
                    ),
                ),
            ],
            options={
                "verbose_name": "Interpretation Quiz Answer",
                "verbose_name_plural": "Interpretation Quiz Answers",
                "ordering": ("order", "id"),
            },
        ),
        migrations.RunPython(copy_denormalized_data, migrations.RunPython.noop),
        migrations.AddConstraint(
            model_name="interpretationquizconclusion",
            constraint=models.UniqueConstraint(
                fields=("quiz", "symbol"),
                name="uniq_interpretation_conclusion_symbol_per_quiz",
            ),
        ),
    ]
